/*
 * This file is part of the GROMACS molecular simulation package.
 *
 * Copyright (c) 2014, by the GROMACS development team, led by
 * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
 * and including many others, as listed in the AUTHORS file in the
 * top-level source directory and at http://www.gromacs.org.
 *
 * GROMACS is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * GROMACS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with GROMACS; if not, see
 * http://www.gnu.org/licenses, or write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * If you want to redistribute modifications to GROMACS, please
 * consider that scientific software is very special. Version
 * control is crucial - bugs must be traceable. We will be happy to
 * consider code for inclusion in the official distribution, but
 * derived work must not be called official GROMACS. Details are found
 * in the README & COPYING files - if they are missing, get the
 * official version at http://www.gromacs.org.
 *
 * To help us fund GROMACS development, we humbly ask that you cite
 * the research papers on the package. Check out http://www.gromacs.org.
 */
/*! \internal \file
 * \brief
 * Implements classes from cmdlinemodule.h.
 *
 * \author Teemu Murtola <teemu.murtola@gmail.com>
 * \ingroup module_commandline
 */
#include "gmxpre.h"

#include "cmdlinemodule.h"

#include "gromacs/commandline/cmdlinehelpcontext.h"

namespace gmx
{

class CommandLineModuleSettings::Impl
{
    public:
        Impl() : defaultNiceLevel_(19) {}

        int defaultNiceLevel_;
};

CommandLineModuleSettings::CommandLineModuleSettings()
    : impl_(new Impl)
{
}

CommandLineModuleSettings::~CommandLineModuleSettings()
{
}

int CommandLineModuleSettings::defaultNiceLevel() const
{
    return impl_->defaultNiceLevel_;
}

void CommandLineModuleSettings::setDefaultNiceLevel(int niceLevel)
{
    impl_->defaultNiceLevel_ = niceLevel;
}

//! \cond libapi
void writeCommandLineHelpCMain(
        const CommandLineHelpContext &context, const char *name,
        int (*mainFunction)(int argc, char *argv[]))
{
    char *argv[2];
    int   argc = 1;
    // TODO: The constness should not be cast away.
    argv[0] = const_cast<char *>(name);
    argv[1] = NULL;
    GlobalCommandLineHelpContext global(context);
    mainFunction(argc, argv);
}
//! \endcond

} // namespace gmx
